@@ -21,7 +21,7 @@ from twisted.plugin import IPlugin
 from twisted.python import failure
 from zope import interface
 
-from piped import exceptions, graph, log, util, conf, contrib, plugin, resource, dependencies, processors as piped_processors
+from piped import exceptions, graph, log, util, conf, resource, dependencies, processors as piped_processors, plugin, plugins
 
 
 class IProcessor(IPlugin):
@@ -294,22 +294,28 @@ class _ProcessTracer(object):
     
     """
 
-    def __init__(self, token_value):
+    def __init__(self):
         self.traced = list()
-        self.token_value = token_value
-        self.token_name = 'trace_token'
+        self.token_value = uuid.uuid4().get_hex()
+        self.token_name = 'trace_token_'+self.token_value
 
         self.sentinel = object()
         self.encoder = util.BatonJSONEncoder()
 
-    def find_trace_token(self, frame):
-        """ Returns the value of the trace token if it is in the current stack of frames. """
-        if self._is_the_trace_token_in_the_current_stack(frame):
+    def refresh_trace_token(self, frame):
+        """ Attempt to find and refresh the trace token in the given stack of frames.
+
+        :return: The value of the trace token value if it is found.
+        """
+        if self._refresh_trace_token_if_found_in_the_current_stack(frame):
             return self.token_value
 
     def add_trace_entry(self, frame, kind, **state):
-        """ Add an entry to the trace if the correct trace token is in the current stack of frames. """
-        if not self._is_the_trace_token_in_the_current_stack(frame):
+        """ Add an entry to the trace if the correct trace token is in the current stack of frames.
+
+        :return: The state added to the trace.
+        """
+        if not self._refresh_trace_token_if_found_in_the_current_stack(frame):
             return
 
         if state['source'] is None:
@@ -324,18 +330,16 @@ class _ProcessTracer(object):
         elif kind in ('process_source_raised', '_process_consumer_raised', '_process_error_consumer', '_process_error_consumer_raised'):
             state['failure'] = self._get_trace_failure()
 
-            if kind == 'process_source_raised':
-                state['destination'] = self._find_nearest_possible_source(frame)
-
         self._append_to_trace(**state)
 
+        return state
+
     def _find_nearest_possible_source(self, frame):
         """ Returns the first 'self' variable in a stack of frames that is not part of piped.processing """
         while frame:
             possible_source = frame.f_locals.get('self', None)
-            if possible_source and not isinstance(possible_source, (TwistedProcessorGraphEvaluator, failure.Failure)):
+            if possible_source and not isinstance(possible_source, (TwistedProcessorGraphEvaluator, failure.Failure, defer.Deferred)):
                 return possible_source
-
             frame = frame.f_back
 
     def _get_trace_failure(self):
@@ -347,14 +351,25 @@ class _ProcessTracer(object):
             traceback = reason.getTraceback(elideFrameworkCode=True, detail='brief')
         )
 
-    def _is_the_trace_token_in_the_current_stack(self, frame):
-        """ Looks for the trace token in a stack of frames. """
-        # Look for the sentinel in the stack.
+    def _refresh_trace_token_if_found_in_the_current_stack(self, frame):
+        """ Looks for the trace token in a stack of frames and copies it to all frames
+        leading from the frame the token was found in to the given frame.
+
+        :return: True if the token was found, False otherwise.
+        """
+
+        frames_visited = list()
+
         while frame:
             # It may be in the locals of the current frame
             if frame.f_locals.get(self.token_name, self.sentinel) == self.token_value:
+                # if it is found, add it to the locals of the intermediary frames
+                for visited_frame in frames_visited:
+                    visited_frame.f_locals[self.token_name] = self.token_value
+
                 return True
 
+            frames_visited.append(frame)
             frame = frame.f_back
 
         return False
@@ -386,23 +401,29 @@ class TwistedProcessorGraphEvaluator(object):
     def _add_trace_entry(cls, *a, **kw):
         for tracer in cls.tracers:
             frame = sys._getframe(1)
-            tracer.add_trace_entry(frame, *a, **kw)
+            entry = tracer.add_trace_entry(frame, *a, **kw)
+            if entry:
+                return entry
+
 
     @classmethod
-    def _find_trace_token(cls):
+    def _refresh_trace_token(cls):
         for tracer in cls.tracers:
             frame = sys._getframe(1)
-            token = tracer.find_trace_token(frame)
+            token = tracer.refresh_trace_token(frame)
             if token:
                 return token
+            else:
+                pass
 
     @defer.inlineCallbacks
     def _process(self, processor, baton, results):
-        # Get a copy of the trace token so that it is easier to find by the tracer.
+        # Refresh the trace token, which will copy the trace token as far down the stack
+        # as possible, so that it is easier to find by the tracer.
         # This fixes a corner case where a processor performs some deep magic that would
         # change the stack, making it hard (or even impossible) to locate the original
         # trace token location.
-        trace_token = self._find_trace_token()
+        self._refresh_trace_token()
 
         try:
             # Profile the time each processor spends.
@@ -457,23 +478,33 @@ class TwistedProcessorGraphEvaluator(object):
         results = list()
 
         for source_processor in self.processor_graph.sources:
+            # store the trace entry before processing in order to use it as a destination if the processing raises an Exception
+            entry = self._add_trace_entry('process_source', source=None, destination=source_processor, baton=baton)
             try:
-                self._add_trace_entry('process_source', source=None, destination=source_processor, baton=baton)
                 yield self._process(source_processor, baton, results)
             except Exception as e:
-                self._add_trace_entry('process_source_raised', source=source_processor, destination=None, baton=baton)
+                source = entry['source'] if entry else None
+                self._add_trace_entry('process_source_raised', source=source_processor, destination=source, baton=baton)
                 raise
 
         defer.returnValue(results)
 
+    # Calling the evaluator directly should be the same as starting to process a baton in a pipeline:
+    __call__ = process
+
     @defer.inlineCallbacks
     def traced_process(self, *a, **kw):
         """ Traces and processes a baton asynchronously through a processor graph.
 
+        .. warn:: The act of tracing introduces a new variable in the locals of
+            all the traced frames on the form ``trace_token_UUID4``.
+
         .. seealso:: :class:`_ProcessTracer` and :meth:`process`.
         """
-        trace_token = uuid.uuid4().get_hex()
-        tracer = _ProcessTracer(token_value=trace_token)
+        tracer = _ProcessTracer()
+        # add the token to the current locals, so _refresh_trace_token may find it
+        locals()[tracer.token_name] = tracer.token_value
+
         try:
             self.tracers.append(tracer)
             results = yield self.process(*a, **kw)
@@ -483,7 +514,6 @@ class TwistedProcessorGraphEvaluator(object):
             self.tracers.remove(tracer)
         defer.returnValue((results, tracer.traced))
 
-
     def configure_processors(self, runtime_environment):
         # give the processor a reference to its evaluator
         for processor in self.processor_graph:
@@ -522,7 +552,7 @@ class ProcessorGraphFactory(object):
         self.pipelines_configuration = self._get_pipeline_configuration(runtime_environment)
 
         if not self.pipelines_configuration:
-            log.warn('Could not find any pipeline definitions in the configuration.')
+            log.info('No pipeline definitions were found in the configuration.')
 
         self.plugin_manager = ProcessorPluginManager()
         self.plugin_manager.configure(runtime_environment)
@@ -545,7 +575,7 @@ class ProcessorGraphFactory(object):
         # attempt to convert the configuration to a base format before checking if it is a valid configuration.
         config = self._as_processor_configuration(maybe_configuration)
         if isinstance(config, dict):
-            if 'processor' in config or 'inline-pipeline' in config:
+            if '__processor__' in config or 'inline-pipeline' in config:
                 return True
         return False
 
@@ -632,7 +662,7 @@ class ProcessorGraphFactory(object):
 
         Processors can be defined in several ways, the base format being:
 
-            processor: processor-name
+            __processor__: processor-name
             key1: value1
             key2: value2
 
@@ -658,10 +688,10 @@ class ProcessorGraphFactory(object):
 
         if isinstance(processor_configuration, basestring):
             # if it is a string, we assume it is the name of a processor
-            return dict(processor=processor_configuration)
+            return dict(__processor__=processor_configuration)
 
         if isinstance(processor_configuration, dict):
-            if 'processor' in processor_configuration or 'inline-pipeline' in processor_configuration:
+            if '__processor__' in processor_configuration or 'inline-pipeline' in processor_configuration:
                 # it is already a processor configuration
                 return processor_configuration
 
@@ -681,7 +711,7 @@ class ProcessorGraphFactory(object):
             del processor_configuration[processor_name]
 
             # update the processor name and configuration
-            processor_configuration['processor'] = processor_name
+            processor_configuration['__processor__'] = processor_name
             processor_configuration.update(processor_arguments)
 
             return processor_configuration
@@ -695,7 +725,7 @@ class ProcessorGraphFactory(object):
     def _fail_if_pipeline_is_misconfigured(cls, pipeline):
         # the pipeline configuration might not be iterable...
         if isinstance(pipeline, collections.Iterable):
-            if any(key in pipeline for key in ('processor', 'inline-pipeline', 'chained_consumers', 'consumers', 'existing')):
+            if any(key in pipeline for key in ('__processor__', 'inline-pipeline', 'chained_consumers', 'consumers', 'existing')):
                 return
 
         e_msg = 'invalid contents of pipeline-configuration'
@@ -740,7 +770,7 @@ class ProcessorGraphFactory(object):
                 # Sub-pipelines can be nested in sub-consumers.
                 self._flatten_pipeline(pipeline_name, attribute)
 
-                if 'processor' in attribute or 'existing' in attribute:
+                if '__processor__' in attribute or 'existing' in attribute:
                     attribute_after_flattening.append(attribute)
                     # Any potential sub-pipelines in consumers have been flattened above,
                     # so we're done here.
@@ -846,7 +876,7 @@ class ProcessorGraphFactory(object):
             processor_queue.extendleft(current_processor.get('error_consumers', tuple()))
 
             next_in_queue = overrides[0] # Peek
-            if current_processor['processor'] == next_in_queue['processor']:
+            if current_processor['__processor__'] == next_in_queue['__processor__']:
                 cls._override_processor_options(current_processor, overrides.popleft())
 
         cls._fail_if_not_all_overrides_were_used(overrides, pipeline_name)
@@ -902,7 +932,7 @@ class ProcessorGraphFactory(object):
         if not overrides:
             return
 
-        names_of_processors = [override['processor'] for override in overrides]
+        names_of_processors = [override['__processor__'] for override in overrides]
         e_msg = 'overrides provided but not used: %s' % (names_of_processors, )
         detail = ('The pipeline "%s" was provided with overrides to processors not found in the pipeline it '
                   'inherited from.') % (pipeline_name, )
@@ -944,7 +974,7 @@ class ProcessorGraphFactory(object):
 
         # We pop away stuff because we want the remaining dictionary to define the
         # configuration values.
-        plugin_name = copied_processor_configuration.pop('processor')
+        plugin_name = copied_processor_configuration.pop('__processor__')
         processor_id = copied_processor_configuration.pop('id', None)
 
         plugin_factory = self._get_plugin_factory_or_fail(plugin_name)
@@ -1204,6 +1234,6 @@ class RuntimeEnvironment(object):
 
 
 class ProcessorPluginManager(plugin.PluginManager):
-    plugin_packages = [piped_processors, contrib]
+    plugin_packages = [piped_processors, plugins]
     plugin_interface = IProcessor
     plugin_configuration_name = 'processors'
