@@ -11,6 +11,7 @@ import xmlrpclib
 import copy
 import json
 
+import twisted
 from twisted.internet import defer, reactor
 from twisted.application import service
 from twisted.python import failure, filepath, reflect
@@ -156,7 +157,7 @@ def dict_set_path(dict_like, path_or_list, value, separator='.'):
     key = keys[-1]
     try:
         dict_like[key] = value
-    except (TypeError, ValueError):
+    except (TypeError, ValueError, AttributeError):
         # Not a dict, so try setting an attribute. If that fails, give up.
         if hasattr(dict_like, key):
             setattr(dict_like, key, value)
@@ -509,33 +510,33 @@ class BatonJSONEncoder(PipedJSONEncoder):
             return repr(obj)
 
 
-class PullFromQueueAndProcessInPipeline(service.Service):
+class PullFromQueueAndProcessWithDependency(service.Service):
     _waiting_on_queue = None
-    _waiting_on_pipeline = None
+    _waiting_on_processor = None
 
-    def __init__(self, deferred_queue, pipeline_name):
+    def __init__(self, deferred_queue, dependency_config):
         self.deferred_queue = deferred_queue
-        self.pipeline_name = pipeline_name
+        self.dependency_config = dict(provider=dependency_config) if isinstance(dependency_config, basestring) else dependency_config
 
     def configure(self, runtime_environment):
+        self.runtime_environment = runtime_environment
         dependency_manager = runtime_environment.dependency_manager
 
-        self._pipeline_dependency = dependency_manager.as_dependency(dict(provider='pipeline.%s'%self.pipeline_name))
-        dependency_manager.add_dependency(self, self._pipeline_dependency)
+        self._processor_dependency = dependency_manager.add_dependency(self, self.dependency_config)
 
     def startService(self):
         service.Service.startService(self)
 
-        if self._waiting_on_pipeline:
+        if self._waiting_on_processor:
             # We've been started, stopped, started again, and there's
-            # pipeline-processing going on. The process-loop will find
+            # processing going on. The process-loop will find
             # self.running to be true, so let it continue instead of
             # having multiple processing loops.
             return
 
         self._process_input()
 
-        log.info('Starting pulling: %s into %s.'%(self.name, self.pipeline_name))
+        log.info('Starting pulling: %s using %s.'%(self.name, self.dependency_config['provider']))
 
     def stopService(self):
         service.Service.stopService(self)
@@ -558,20 +559,90 @@ class PullFromQueueAndProcessInPipeline(service.Service):
                 self._waiting_on_queue = None
 
             try:
-                self._waiting_on_pipeline = self._pipeline_dependency.wait_for_resource()
-                pipeline = yield self._waiting_on_pipeline
-                self._waiting_on_pipeline = pipeline.process(baton)
-                yield self._waiting_on_pipeline
+                self._waiting_on_processor = self._processor_dependency.wait_for_resource()
+                processor = yield self._waiting_on_processor
+                self._waiting_on_processor = processor(baton)
+                yield self._waiting_on_processor
             except Exception:
                 # An exception occurred while processing the baton, and we don't have anyone to inform
                 # so just log the exception and continue
                 log.error()
             finally:
-                self._waiting_on_pipeline = None
+                self._waiting_on_processor = None
 
 
-class NonCleaningFailure(failure.Failure):
+# we store a reference to the superclass of NonCleaningFailure here because it may change
+# due to the monkey-patching performed when piped is started with the -D argument
+NonCleaningFailureSuperClass = failure.Failure
+
+class NonCleaningFailure(NonCleaningFailureSuperClass):
     """ A Failure subclass that doesn't replace its traceback with repr'd objects. """
 
+    def __init__(self, *a, **kw):
+        # default to capturing vars (locals/globals) in frames when using this class, since
+        # the user most likely isn't worried about the additional cost. this is the default
+        # in twisted <= 11.0
+        if (twisted.version.major, twisted.version.minor) > (11, 0):
+            kw.setdefault('captureVars', True)
+
+        NonCleaningFailureSuperClass.__init__(self, *a, **kw)
+
     def cleanFailure(self):
         pass
+
+
+def create_deferred_state_watcher(obj, attribute_name='_currently'):
+    """
+    Creates a function that when called with a deferred, assigns the deferred to the
+    specified attribute on the target object. When the deferred callbacks or errbacks,
+    the attribute is cleared.
+
+    This can be used to implement functions that should only be executed by one concurrent
+    invocation:
+
+        >>> class A(object):
+        ...     running = None # stores the currently running state
+        ...
+        ...     def run(self):
+        ...         if self.running:
+        ...             # in this example we return here, but we could have called
+        ...             # ``self.running.cancel()`` to cancel the previous invocation and continued.
+        ...             print 'already running'
+        ...             return
+        ...
+        ...         # create the watcher.
+        ...         currently = create_deferred_state_watcher(self, 'running')
+        ...
+        ...         return currently(wait(0))
+        ...
+        >>> a = A()
+        >>> d = a.run()
+        >>> a.run()
+        already running
+    """
+    def clear(result):
+        setattr(obj, attribute_name, None)
+        return result
+
+    def wrapper(retval):
+        if isinstance(retval, defer.Deferred):
+            setattr(obj, attribute_name, retval)
+            retval.addBoth(clear)
+
+        return retval
+
+    return wrapper
+
+
+def fail_after_delay(delay, exception):
+    """ Returns a Deferred that will errback with `exception` after `delay`. """
+    d = defer.Deferred()
+    reactor.callLater(delay, d.errback, exception)
+    return d
+
+
+def wait_for_first(ds):
+    """ Returns a deferred that is callbacked/errbacked with whatever deferred in `ds` fires first. """
+    d = defer.DeferredList(ds, fireOnOneCallback=True, fireOnOneErrback=True, consumeErrors=True)
+    d.addCallback(operator.itemgetter(0))
+    return d
